@@ -1,14 +1,16 @@
 """
     请求的处理
 """
+import random
+import time
+
 import urllib3
-import requests
 from palp import settings
 from typing import Callable
-from urllib.parse import urlparse
+from palp.network.response import Response
 from palp.tool.user_agent import random_ua
 from requests.cookies import RequestsCookieJar
-from palp.network.response import ResponseDownloader
+from palp.tool.short_module import import_module
 from urllib3.exceptions import InsecureRequestWarning
 
 # 禁用不安全警告
@@ -40,16 +42,52 @@ class Request:
     __PALP_ATTRS__ = [
         'filter_repeat',
         'keep_session',
+        'keep_cookie',
+        'new_session',
         'callback',
-        'session',
-        'cookie_jar'
+        'cookie_jar',
+        'priority',
+        'command'
     ]
 
-    def __init__(self, url: str = None, method: str = None, params=None, data=None, headers=None, cookies=None,
-                 files=None, auth=None, timeout=None, allow_redirects=True, proxies=None, hooks=None, stream=None,
-                 verify=None, cert=None, json=None, filter_repeat: bool = None, keep_session: bool = None,
-                 callback: Callable = None, level: int = None, session: requests.Session = None,
-                 cookie_jar: RequestsCookieJar = None, **kwargs):
+    # 下载器
+    DOWNLOADER = None
+    DOWNLOADER_PARSER = None
+
+    def __new__(cls, *args, **kwargs):
+        """
+        导入下载器
+
+        :param args:
+        :param kwargs:
+        """
+        if cls.DOWNLOADER is None:
+            cls.DOWNLOADER = import_module(settings.RESPONSE_DOWNLOADER, instantiate=False)[0]
+            cls.DOWNLOADER_PARSER = import_module(settings.RESPONSE_DOWNLOADER_PARSER, instantiate=False)[0]
+
+        return object.__new__(cls)
+
+    def __init__(
+            self,
+            url: str,
+            method: str = None,
+            params=None,
+            data=None,
+            headers=None,
+            cookies=None,
+            timeout=None,
+            proxies=None,
+            json=None,
+            downloader=None,
+            filter_repeat: bool = False,
+            keep_session: bool = False,
+            keep_cookie: bool = False,
+            callback: Callable = None,
+            cookie_jar: RequestsCookieJar = None,
+            priority: int = settings.DEFAULT_QUEUE_PRIORITY,
+            command: dict = None,
+            **kwargs
+    ):
         """
         requests 参数
         :param url: 请求链接
@@ -57,26 +95,20 @@ class Request:
         :param data: data
         :param headers: 请求头
         :param cookies: cookies
-        :param files: 文件：{ file_name:open(file) }
-        :param auth: 身份验证
         :param timeout: 超时时间
-        :param allow_redirects: 允许重定向：默认 True
         :param proxies: 代理：{'http':'http://ip:port','https':'https://ip:port'}
-        :param hooks: 钩子：{'response':func}
-        :param stream: 流传输
-        :param verify: 证书验证：默认 False
-        :param cert: ssl 证书路径
         :param json: json：也可以使用 data = json.dumps(data)
 
         Palp 参数
-        :param filter_repeat: 是否过滤请求
-        :param keep_session: 是否保持 session
+        :param filter_repeat: 是否过滤请求，settings.FILTER_REQUEST 启用生效，默认 False
+        :param keep_session: 是否保持 session，默认 False
+        :param keep_cookie: 不使用 session 时保持 cookie，默认 False
         :param callback: 回调函数
-        :param level: 启用优先级队列时的优先级，分数越大，优先级约低
+        :param priority: 启用优先级队列时的优先级，分数越大，优先级约低，默认 settings.DEFAULT_QUEUE_PRIORITY
+        :param command: 自定义操作命令，用于自定义 downloader 时使用
 
         Palp 参数（非用户设置）
-        :param session: session 保证每个线程都是独立的
-        :param cookie_jar: cookie_jar，存储 cookie
+        :param cookie_jar: cookie_jar，存储 cookie，这里使用的是 requests 模块的，其它请求的话可以自己提取
 
         传递的参数
         :param kwargs: 其余需要传递的参数，若参数名不存在则返回 None
@@ -84,37 +116,66 @@ class Request:
 
         # Request 所需字段
         self._requests_params = {}  # request 参数
-        self.keep_session = keep_session or False
-        self.filter_repeat = filter_repeat or False
+        self.command = command
         self.callback = callback
-        self.session = session or requests.session()
-        self.level = level or 100
-        self.cookie_jar = cookie_jar or RequestsCookieJar()
+        self.priority = priority
+        self.downloader = downloader
+        self.cookie_jar = cookie_jar
+        self.keep_cookie = keep_cookie
+        self.keep_session = keep_session
+        self.filter_repeat = filter_repeat
 
         # requests 请求参数
         self.url = url
+        self.data = data
+        self.json = json
         self.method = method
         self.params = params
-        self.data = data
-        self.headers = headers or {}
+        self.headers = headers
         self.cookies = cookies
-        self.files = files
-        self.auth = auth
-        self.timeout = timeout or settings.REQUEST_TIMEOUT or 60
-        self.allow_redirects = allow_redirects
-        if not proxies and settings.REQUEST_PROXIES_TUNNEL_URL:
-            proxies = {'http': settings.REQUEST_PROXIES_TUNNEL_URL, 'https': settings.REQUEST_PROXIES_TUNNEL_URL}
+        self.timeout = timeout
         self.proxies = proxies
-        self.hooks = hooks
-        self.stream = stream
-        self.verify = verify
-        self.cert = cert
-        self.json = json
 
         # 传递参数更新一下
         for key, value in kwargs.items():
-            self.__dict__[key] = value
+            self[key] = value
 
+    def send(self) -> Response:
+        """
+        获取响应
+
+        :return: ResponseParser 的解析器
+        """
+        # 加载默认值
+        self.set_default()
+
+        # 判断是否等待
+        if isinstance(settings.REQUEST_DELAY, int) or isinstance(settings.REQUEST_DELAY, float):
+            time.sleep(settings.REQUEST_DELAY)
+        elif isinstance(settings.REQUEST_DELAY, list):
+            time.sleep(random.choice([i for i in range(settings.REQUEST_DELAY[0], settings.REQUEST_DELAY[1] + 1)]))
+
+        # 获取响应
+        response = self.downloader(
+            **self._requests_params,
+            keep_session=self.keep_session,
+            keep_cookie=self.keep_cookie,
+            cookie_jar=self.cookie_jar,
+            command=self.command
+        ).response()
+
+        response_parser = self.__class__.DOWNLOADER_PARSER(response)  # 解析器解析响应
+        if response_parser.cookies:
+            self.cookie_jar.update(response_parser.cookies)  # 将响应的 cookie 更新到 cookieJar
+
+        return response_parser
+
+    def set_default(self):
+        """
+        设置一些默认值
+
+        :return:
+        """
         # 判断类型
         if self.method:
             self.method = self.method.upper()
@@ -123,7 +184,22 @@ class Request:
         else:
             self.method = 'GET'
 
-        # 添加 ua
+        # 设置默认
+        if self.downloader is None:
+            self.downloader = self.__class__.DOWNLOADER
+        if self.cookie_jar is None:
+            self.cookie_jar = RequestsCookieJar()
+        if self.command is None:
+            self.command = {}
+        if self.headers is None:
+            self.headers = {}
+        if self.cookies is None:
+            self.cookies = {}
+        if self.timeout is None:
+            self.timeout = settings.REQUEST_TIMEOUT or 60
+        self._requests_params.setdefault('verify', False)
+
+        # 添加 ua（模块在使用 ua 的时候可能访问降速）
         if settings.RANDOM_USERAGENT:
             ua = random_ua()
         else:
@@ -136,52 +212,6 @@ class Request:
         elif self.headers.get('user-agent'):
             self.headers.update({'user-agent': ua})
 
-    def send(self):
-        """
-
-        :return: ResponseParser 的解析器
-        """
-        response = ResponseDownloader(
-            keep_session=self.keep_session,
-            session=self.session,
-            cookie_jar=self.cookie_jar
-        ).response(**self._requests_params)
-
-        return self.callback, response
-
-    def add_proxy(self, proxies: str = None, allow_domains: list = None) -> None:
-        """
-        指定域名添加代理
-
-        代理选用优先级：proxies > self.proxies > settings.REQUEST_PROXIES_TUNNEL_URL
-
-        使用方式：
-            1、在 RequestMiddleware 中的 request_in 中建立 allow_domains 列表
-            2、使用 request.add_proxy(allow_domains=allow_domains)
-
-        :param proxies: 代理
-        :param allow_domains: 允许的域名（middleware 中使用）
-        :return:
-        """
-        if isinstance(self.proxies, tuple):
-            self.proxies = dict(self.proxies)
-        if isinstance(self.proxies, dict):
-            proxies_input = list(self.proxies.values())[0]
-        else:
-            proxies_input = self.proxies
-
-        proxies = proxies or proxies_input or settings.REQUEST_PROXIES_TUNNEL_URL
-        if not proxies:
-            return
-
-        if allow_domains:
-            if self.domain in allow_domains:
-                self.proxies = {'http': proxies, 'https': proxies}
-            else:
-                self.proxies = None
-        else:
-            self.proxies = {'http': proxies, 'https': proxies}
-
     def to_dict(self) -> dict:
         """
         获取字典形式
@@ -198,21 +228,12 @@ class Request:
         request_dict = {}
 
         for key, value in self.__dict__.items():
-            if key.startswith('_'):
+            if key.startswith('_') or not value:
                 continue
             request_dict[key] = value
 
         return request_dict
 
-    @property
-    def domain(self) -> str:
-        """
-        获取请求的域名
-
-        :return:
-        """
-        return urlparse(self.url).netloc
-
     def __getattr__(self, item):
         """
         可以 request.xxx 进行访问，但是这里严格一点没有就报错避免误导
@@ -245,7 +266,7 @@ class Request:
         :param other:
         :return:
         """
-        return self.level < other.level
+        return self.priority < other.priority
 
     def __str__(self):
         return f"<Request {self.method}-{self.url}>"
