@@ -23,7 +23,16 @@ class DistributiveSpider(Spider):
     """
 
     def __new__(cls, *args, **kwargs):
-        setattr(settings, 'SPIDER_TYPE', 2)
+        """
+        加载 spider 设置，并定义 SPIDER_TYPE
+
+        :param args:
+        :param kwargs:
+        """
+        if not cls.SPIDER_MIDDLEWARE:
+            setattr(settings, 'SPIDER_TYPE', 2)
+            cls.from_settings()
+
         return object.__new__(cls)
 
     def __init__(self, thread_count=None, redis_key: str = None, request_filter=False, item_filter=False):
